@@ -1,31 +1,35 @@
-require 'rubygems'
 
-require 'rubyforge'
+$HERE = File.dirname(__FILE__)
+require "#{$HERE}/echoe/platform"
 
 require 'rake'
 require 'rake/clean'
-require 'rake/contrib/sshpublisher'
+require "#{$HERE}/../vendor/rake/lib/rake/contrib/compositepublisher"
+require "#{$HERE}/../vendor/rake/lib/rake/contrib/sshpublisher"
 require 'rake/gempackagetask'
 require 'rake/rdoctask'
 require 'rake/testtask'
+begin
+require 'spec/rake/spectask'
+rescue LoadError
+end
 require 'rbconfig'
 require 'open-uri'
+
+require "#{$HERE}/echoe/extensions"
+
+require 'rubygems'
+require 'rubygems/specification'
+require "#{$HERE}/echoe/rubygems"
+
+require 'rubyforge'
+require "#{$HERE}/echoe/net"
+require "#{$HERE}/echoe/client"
+
 require 'highline/import'
 
-begin
-  require 'rcov/rcovtask'
-rescue LoadError
-end
-
-begin
-  require 'load_multi_rails_rake_tasks'
-rescue LoadError
-end
-
-$LOAD_PATH << File.dirname(__FILE__)
-require 'echoe/platform'
-require 'echoe/extensions'
-require 'echoe/client'
+begin; require 'rcov/rcovtask'; rescue LoadError; end
+begin; require 'load_multi_rails_rake_tasks'; rescue LoadError; end
 
 =begin rdoc
 
@@ -37,7 +41,7 @@ For example, a simple <tt>Rakefile</tt> might look like this:
 
   Echoe.new("uncapitalizer") do |p|
     p.author = "Evan Weaver"
-    p.summary = "A library that uncapitalizes strings. It's awesome."
+    p.summary = "A library that uncapitalizes strings."
     p.url = "http://www.uncapitalizer.com"
     p.docs_host = "uncapitalizer.com:~/www/files/doc/"
     p.runtime_dependencies = ["string_tools >=1.4.0"]
@@ -69,8 +73,6 @@ Note that you can also set the key and certificate locations in the Rakefile its
 
 Echoe does not force packages to depend on Echoe itself. Instead, it generates a <tt>gemspec</tt> from your <tt>Rakefile</tt> and includes that. Downstream repackagers can use the <tt>gemspec</tt> as-is to build new versions of your gem even without Echoe.
 
-However, Echoe is added to the <tt>development_dependencies</tt> array so that users can automatically install it via <tt>gem install --development</tt> if they prefer. You can override this behavior by setting <tt>p.development_dependencies = []</tt>.
-
 == Cross-packaging
 
 Echoe supports platform Rake targets to allow you to cross-package your gems. Just write the spec assuming <tt>RUBY_PLATFORM</tt> will be what you need it to be for each architecture, and then invoke Rake with the platform name when you're cross-packaging.
@@ -111,6 +113,8 @@ Testing options:
 
 * <tt>clean_pattern</tt> - A filename array, glob array, or regex for files that should be removed when <tt>rake clean</tt> is run.
 * <tt>test_pattern</tt> - A filename array, glob array, or regex for test runners. Overridden by <tt>"test/test_all.rb"</tt>, if it exists.
+
+* <tt>spec_pattern</tt> - A filename array, glob array, or regex for test runners.
 * <tt>rcov_options</tt> - Any extra flags to pass to RCov when coverage reports are run.
 
 Uncommon packaging options:
@@ -148,10 +152,10 @@ Documentation options:
 class Echoe
 
   # user-configurable
-  attr_accessor :author, :changes, :clean_pattern, :description, :email, :runtime_dependencies, :development_dependencies, :need_tgz, :need_tar_gz, :need_gem, :need_zip, :rdoc_pattern, :project, :summary, :test_pattern, :url, :version, :docs_host, :rdoc_template, :manifest_name, :install_message, :extension_pattern, :private_key, :certificate_chain, :require_signed, :ruby_version, :platform, :ignore_pattern, :executable_pattern, :changelog, :rcov_options, :gemspec_format
+  attr_accessor :author, :changes, :clean_pattern, :description, :email, :runtime_dependencies, :development_dependencies, :need_tgz, :need_tar_gz, :need_gem, :need_zip, :rdoc_pattern, :project, :summary, :test_pattern, :spec_pattern, :url, :version, :docs_host, :rdoc_template, :manifest_name, :install_message, :extension_pattern, :private_key, :certificate_chain, :require_signed, :ruby_version, :platform, :ignore_pattern, :executable_pattern, :changelog, :rcov_options, :gemspec_format
 
   # best left alone
-  attr_accessor :name, :lib_files, :test_files, :bin_files, :spec, :rdoc_options, :rubyforge_name, :has_rdoc, :include_gemspec, :include_rakefile, :gemspec_name, :retain_gemspec, :rakefile_name, :eval, :files, :changelog_patterns, :rubygems_version, :use_sudo
+  attr_accessor :name, :lib_files, :test_files, :bin_files, :spec, :rdoc_options, :rubyforge_name, :has_rdoc, :include_gemspec, :include_rakefile, :gemspec_name, :retain_gemspec, :rakefile_name, :eval, :files, :changelog_patterns, :rubygems_version, :use_sudo, :gem_bin
 
   # legacy
   attr_accessor :extra_deps, :rdoc_files, :extensions, :dependencies
@@ -165,8 +169,9 @@ class Echoe
     self.url = ""
     self.author = ""
     self.email = ""
-    self.clean_pattern = ["pkg", "doc", 'build/*', '**/coverage', '**/*.o', '**/*.so', '**/*.a', 'lib/*-*', '**/*.log', "{ext,lib}/*.{bundle,so,obj,pdb,lib,def,exp}", "ext/Makefile", "{ext,lib}/**/*.{bundle,so,obj,pdb,lib,def,exp}", "ext/**/Makefile", "pkg", "*.gem", ".config"]
+    self.clean_pattern = ["pkg", "doc", 'build/*', '**/coverage', '**/*.o', '**/*.so', '**/*.a', '**/*.log', "{ext,lib}/*.{bundle,so,obj,pdb,lib,def,exp}", "ext/Makefile", "{ext,lib}/**/*.{bundle,so,obj,pdb,lib,def,exp}", "ext/**/Makefile", "pkg", "*.gem", ".config"]
     self.test_pattern = File.exist?("test/test_all.rb") ? "test/test_all.rb" : ['test/**/test_*.rb', 'test/**/*_test.rb']
+    self.spec_pattern = "spec/**/*_spec.rb"
     self.ignore_pattern = /^(pkg|doc)|\.svn|CVS|\.bzr|\.DS|\.git/
 
     self.changelog_patterns = {
@@ -185,7 +190,8 @@ class Echoe
     self.install_message = nil
     self.executable_pattern = /^bin\//
     self.has_rdoc = true
-    self.use_sudo = RUBY_PLATFORM !~ /mswin32|cygwin/
+    self.use_sudo = !Platform.windows?
+    self.gem_bin = "gem#{Platform.suffix}"
     self.rcov_options = []
     self.rdoc_pattern = /^(lib|bin|tasks|ext)|^README|^CHANGELOG|^TODO|^LICENSE|^COPYING$/
 
@@ -198,7 +204,7 @@ class Echoe
     self.rdoc_options += ['--main', readme] if readme
 
     self.runtime_dependencies = []
-    self.development_dependencies = ["echoe"]
+    self.development_dependencies = [] # These appear to not work at all
     self.manifest_name = "Manifest"
     self.extension_pattern = ["ext/**/extconf.rb", "ext/extconf.rb"]
     self.private_key = ENV['GEM_PRIVATE_KEY']
@@ -229,7 +235,7 @@ class Echoe
 
     # read manifest
     begin
-      self.files = File.read(manifest_name).split +
+      self.files = File.readlines(manifest_name).map { |x| x.strip } +
         [(gemspec_name if include_gemspec)] +
         [(rakefile_name if include_rakefile)]
       self.files = files.compact.uniq
@@ -258,14 +264,16 @@ class Echoe
       end
     end
 
-    self.changes = if File.exist? changelog
-      Array(changelog_patterns[:changes]).map do |pattern|
-        open(changelog) do |log|
-          log.read[pattern, 1]
-        end
-      end.compact.first or ""
-    else
-      ""
+    unless self.changes
+      self.changes = if File.exist? changelog
+        Array(changelog_patterns[:changes]).map do |pattern|
+          open(changelog) do |log|
+            log.read[pattern, 1]
+          end
+        end.compact.first or ""
+      else
+        ""
+      end
     end
 
     # set some post-defaults
@@ -279,6 +287,7 @@ class Echoe
     self.rdoc_pattern = apply_pattern(rdoc_pattern, files) - [manifest_name]
     self.executable_pattern = apply_pattern(executable_pattern, files)
     self.test_pattern = apply_pattern(test_pattern)
+    self.spec_pattern = apply_pattern(spec_pattern)
 
     define_tasks
   end
@@ -381,7 +390,15 @@ class Echoe
       pkg.need_tar_gz = @need_tar_gz
       pkg.need_zip = @need_zip
     end
-
+    
+    desc "Display Echoe's knowledge of your system"
+    task :details do
+      (self.instance_variables.sort - ['@spec']).each do |var|
+        puts "#{var}: #{instance_variable_get(var).inspect}"
+      end
+    end
+    
+    desc "Builds the .gemspec"
     task :build_gemspec do
       # Construct the gemspec file, if needed.
       if include_gemspec
@@ -399,11 +416,15 @@ class Echoe
           end
         end
       end
+      puts "Gemspec generated"
     end
 
     # Chain it to the gemspec task prerequisite
     task gemspec_name.to_sym => [:build_gemspec]
 
+    desc "Generates manifest & gemspec in one go"
+    task :build => [:manifest, :build_gemspec]
+
     task :package do
       # Chain some cleanup tasks to the default :package task.
       # Remove the gemfile if it wasn't actually requested.
@@ -428,19 +449,19 @@ class Echoe
 
     desc 'Install the gem'
     task :install => [:clean, :package, :uninstall] do
-      system "#{'sudo' if use_sudo} gem install pkg/*.gem -P MediumSecurity --no-update-sources"
+      system "#{'sudo' if use_sudo} #{gem_bin} install pkg/*.gem -P MediumSecurity --no-update-sources"
     end
 
     namespace :install do
       desc 'Install the gem including development dependencies'
       task :development => [:clean, :package, :uninstall] do
-        system "#{'sudo' if use_sudo} gem install pkg/*.gem -P MediumSecurity --no-update-sources --development"
+        system "#{'sudo' if use_sudo} #{gem_bin} install pkg/*.gem -P MediumSecurity --no-update-sources --development"
       end
     end
 
     desc 'Uninstall the gem'
     task :uninstall do
-      system "#{'sudo' if use_sudo} gem uninstall #{name} -a -I -x"
+      system "#{'sudo' if use_sudo} #{gem_bin} uninstall #{name} -a -I -x"
     end
 
     desc 'Package and upload the release to Rubyforge'
@@ -469,7 +490,7 @@ class Echoe
                   (@need_gem ? pkg_gem : nil)].compact
 
         puts "Releasing #{name} v. #{version}"
-        self.version = self.version.ljust(3)
+        self.version = self.version.to_s.ljust(3)
 
         rf.add_release project, name, version, *files
       end
@@ -491,7 +512,7 @@ class Echoe
           lib_target = nil
           Dir.chdir(ext_dir) do
             ruby File.basename(extension)
-            system(PLATFORM =~ /win32/ ? 'nmake' : 'make')
+            system(RUBY_PLATFORM =~ /win32/ ? 'nmake' : 'make')
             lib_target = open('Makefile').readlines.grep(/target_prefix = /).first.split('=').last.chomp("\n").strip
           end
           Dir["#{ext_dir}/*.#{Config::CONFIG['DLEXT']}"].each do |file|
@@ -502,7 +523,8 @@ class Echoe
         end
       end
 
-      task :test => [:compile]
+      task :test => [:compile] if test_pattern.any?
+      task :spec => :compile if spec_pattern.any?
 
     end
 
@@ -582,7 +604,15 @@ class Echoe
         File.open(filename, 'w') do |f|
           f.write "Subject: #{name.capitalize} #{version}\n\n"
           f.write "#{name.capitalize} has been updated to #{version}. #{name.capitalize} is #{summary.uncapitalize}\n\n"
-          f.write "Changes in this version: #{changes.sub(/^\s*[\w\d\.]+\s+/, '').uncapitalize}\n\n" unless changes.empty?
+          unless changes.empty?
+            f.write "Changes in this version: "
+            if changes.include?("\n")
+              f.write(changes)
+            else
+              f.write(changes.sub(/^\s*[\w\d\.]+\s+/, '').uncapitalize)
+            end
+            f.write("\n\n")
+          end
           f.write "More information is available at #{url} .\n\n" unless url.empty?
         end
       end
@@ -627,7 +657,7 @@ class Echoe
       puts "Building Manifest"
       old_files = files
       files = []
-      Dir['**/**'].each do |file|
+      Dir['**/**'].sort.each do |file|
         next unless file
         next if ignore_pattern.include?(file)
         next if File.directory?(file)
@@ -690,9 +720,17 @@ class Echoe
         end
       end
 
+      task :default => :test
     end
 
-    task :default => :test
+    if spec_pattern.any?
+      desc "Run the spec suite"
+      Spec::Rake::SpecTask.new('spec') do |t|
+        t.spec_files = spec_pattern
+      end
+
+      task :default => :spec
+    end
 
     if defined? Rcov
       Rcov::RcovTask.new(:coverage) do |t|
